//QR code decorder(QD25) Control Class
//This class has base functions based on the manual
//So, if you want to know mode parameter information,
//please refer the QD25 manual
//
//Programed by Kenichi Ohara, AIST
//2007/08/29 Ver.0.1: Implementation of base functions

#include"QD25Base.h"

void QD25Base::Initialize(char *dev)
{
  com.Initialize(dev, 38400);
}

void QD25Base::Finalize()
{
  com.Finalize();
}

bool QD25Base::SendCommand(char *command)
{
  char mes[200];
  int length;

  char *start = "START\r";
  char *end   = "END\r";

  //Clear message buffer
  bzero(mes,200);

  //Send Data Send START Message
  com.SendMessage(start,strlen(start));
  length=com.Receive(mes);
  if(mes[0] != 'O' ||  mes[1] != 'K'){
    cout<<"ERROR:\"START\" Message"<<std::endl;
    return false;
  }

  //Send Setting Command 
  com.SendMessage(command,strlen(command));
  length=com.Receive(mes);
  if(mes[0] != 'O' || mes[1] != 'K'){
    cout<<mes<<endl;
    return false;
  }

  //Send Data END Message
  com.SendMessage(end,strlen(end));
  length=com.Receive(mes);
  if(mes[0] != 'O' ||  mes[1] != 'K'){
    cout<<"ERROR:\"END\" Message";
    return false;
  }
  
  return true;
}

bool QD25Base::SendCommand(char *command,char *message)
{
  char mes[200];
  int length;

  char *start = "START\r";
  char *end   = "END\r";

  //Clear message buffer
  bzero(mes,200);

  //Send Data Send START Message
  com.SendMessage(start,strlen(start));
  length=com.Receive(mes);
  if(mes[0] != 'O' ||  mes[1] != 'K'){
    cout<<"ERROR:\"START\" Message";
    return false;
  }

  //Send Setting Command 
  com.SendMessage(command,strlen(command));
  length=com.Receive(mes);

  memcpy(message,mes,length);

  //Send Data END Message
  com.SendMessage(end,strlen(end));
  length=com.Receive(mes);
  if(mes[0] != 'O' ||  mes[1] != 'K'){
    cout<<"ERROR:\"END\" Message";
    return false;
  }
  return true;
}

bool QD25Base::SetProtocol(int mode)
{
  char command[100];
  
  sprintf(command,"WR21#%d\r",mode);

  if(SendCommand(command)){
    return false;
  }

  return true;
}

int QD25Base::GetProtocol(){
  char command[100];
  char message[1024];
  int mode;

  sprintf(command,"RD21\r");

  if(!SendCommand(command,message)){
    return -1;
  }
  mode = atoi(message);
  switch(mode){
  case 0:
    cout<<"No Protocol Mode"<<endl;
    break;
  case 1:
    cout<<"ACK/NAK Mode"<<endl;
    break;
  }
  return mode;
}

bool QD25Base::SetBaudRate(int mode)
{
  char command[100];
  
  sprintf(command,"WR22#%d\r",mode);

  if(SendCommand(command)){
    return false;
  }

  return true;
}

int QD25Base::GetBaudRate(){
  char command[100];
  char message[1024];
  int mode;
  sprintf(command,"RD22\r");

  if(!SendCommand(command,message)){
    return -1;
  }
  mode = atoi(message);
  switch(mode){
  case 0:
    cout<<"BaudRate:4800bps"<<endl;
    break;
  case 1:
    cout<<"BaudRate:9600bps"<<endl;
    break;
  case 2:
    cout<<"BaudRate:19200bps"<<endl;
    break;  
  case 3:
    cout<<"BaudRate:38400bps"<<endl;
    break;  
  case 4:
    cout<<"BaudRate:57600bps"<<endl;
    break;
  case 5:
    cout<<"BaudRate:115200bps"<<endl;
    break;
  }
  return mode;
}

bool QD25Base::SetCommunicationParameters(char *com){
  char command[100];

  if(strlen(command) != 3){
    cout<<"Please Set Correct Command!"<<endl;
    return false;
  }
  
  sprintf(command,"WR23#%s\r",com);

  if(SendCommand(command)){
    return false;
  }
  return true;
}

int QD25Base::GetCommunicationParameters(){
  char command[100];
  char message[1024];

  sprintf(command,"RD23\r");

  if(!SendCommand(command,message)){
    return -1;
  }

  //Data Bit
  switch(message[0]){
  case '7':
    cout<<"Data Bit:7"<<endl;
    break;
  case '8':
    cout<<"Data Bit:8"<<endl;
    break;
  }

  //Parity
  cout<<"Parity Mode:";
  switch(message[1]){
  case 'N':
    cout<<"Non Parity"<<endl;
    break;
  case 'E':
    cout<<"Even Parity"<<endl;
    break;
  case 'O':
    cout<<"Odd Parity"<<endl;
    break;
  }
  
  //Stop Bit
  cout<<"Stop Bit";
  switch(message[2]){
  case '1':
    cout<<"1"<<endl;
    break;
  case '2':
    cout<<"2"<<endl;
    break;
  }

  return 0;
}


bool QD25Base::SetHeaderTerminator(int mode){
  char command[100];

  sprintf(command,"WR24#%d\r",mode);

  if(!SendCommand(command)){
    return false;
  }
  return true;
}

int QD25Base::GetHeaderTerminator(){
  char command[100];
  char message[1024];
  int mode;

  sprintf(command,"RD24\r");

  if(!SendCommand(command,message)){
    return -1;
  }

  mode = atoi(message);  

  switch(mode){
  case 0:
    cout<<"Header   :STX"<<endl;
    cout<<"Terminator:ETX"<<endl;
    break;
  case 1:
    cout<<"Header   ::None"<<endl;
    cout<<"Terminator:CR"<<endl;
    break;
  case 2:
    cout<<"Header    :None"<<endl;
    cout<<"Terminator:LF"<<endl;
    break;
  case 3:
    cout<<"Header    :None"<<endl;
    cout<<"Terminator:CR+LF"<<endl;
    break;
  }
  return mode;
}

bool QD25Base::SetBuzzerMode(int mode){
  char command[100];

  sprintf(command,"WR33#%d\r",mode);

  if(!SendCommand(command)){
    cout<<"Cannot Set buzzer mode"<<endl;
    return false;
  }
  return true;
}

int QD25Base::GetBuzzerMode(){
  char command[100];
  char message[1024];
  int mode;

  sprintf(command,"RD33\r");

  if(!SendCommand(command,message)){
    return -1;
  }

  mode = atoi(message);

  switch(mode){
  case 0:
    cout<<"BuzzerMode:Enable"<<endl;
    break;
  case 1:
    cout<<"BuzzerMode:Disable"<<endl;
  }
  return mode;
}

bool QD25Base::SetReadMode(int mode){
  char command[100];

  sprintf(command,"WR01#%d\r",mode);

  if(!SendCommand(command)){
    cout<<"SetReadMode Error"<<endl;
    return false;
  }
  return true;
}

int QD25Base::GetReadMode(){
  char command[100];
  char message[1024];
  int mode;

  sprintf(command,"RD01\r");

  if(!SendCommand(command,message)){
    return -1;
  }

  mode = atoi(message);

  cout<<"Read Mode:";
  switch(mode){
  case 0:
    cout<<"Continuous Read Mode A"<<endl;
    break;
  case 1:
    cout<<"Continuous Read Mode B"<<endl;
    break;
  case 2:
    cout<<"Direct Trigger Mode"<<endl;
    break;
  case 3:
    cout<<"Indirect Trigger Mode"<<endl;
    break;
  case 4:
    cout<<"Software Trigger Mode"<<endl;
    break;
  }
  return mode;
}

bool QD25Base::SoftTriggerOn(){
  char command[100];

  cout<<"SoftTrigger ON"<<endl;
  sprintf(command,"READON\r");
  com.SendMessage(command,strlen(command));
  return true;
}

bool QD25Base::SoftTriggerOff(){
  char command[100];

  sprintf(command,"READOFF\r");

  com.SendMessage(command,strlen(command));
  cout<<"SoftTrigger OFF"<<endl;

  return true;
}

bool QD25Base::SetSynchronousMode(int mode){
  char command[100];

  sprintf(command,"WR11#%d\r",mode);

  if(!SendCommand(command)){
    return false;
  }
  return true;
}

int QD25Base::GetSynchronousMode(){
  char command[100];
  char message[1024];
  int mode;

  sprintf(command,"RD11\r");

  if(!SendCommand(command,message)){
    return -1;
  }

  mode = atoi(message);

  switch(mode){
  case 0:
    cout<<"Asynchronous Mode"<<endl;
    break;
  case 1:
    cout<<"Synchronous Mode"<<endl;
    break;
  }
  return mode;
}

int QD25Base::ReceiveCodeData(char *data){
   char message[1024];
   string checkdata;
   int length;
   bzero(message,1024);
   
   length=com.Receive(message);
   memcpy(data,message,length);
   checkdata = data;
   if(checkdata == "NO MARK\n"){
      return -1;
   }
   return length;
}

bool QD25Base::SetUnreadCheckMode(int mode){
  char command[100];

  sprintf(command,"WR12#%d\r",mode);

  if(!SendCommand(command)){
    cout<<"SetUnreadMode Error"<<endl;
    return false;
  }
  return true;
}

int QD25Base::GetUnreadCheckMode(){
  char command[100];
  char message[1024];
  int mode;

  sprintf(command,"RD12\r");

  if(!SendCommand(command,message)){
    return -1;
  }

  mode = atoi(message);

  cout<<"Unread Check:";
  switch(mode){
  case 0:
    cout<<"Enable"<<endl;
    break;
  case 1:
    cout<<"Disable"<<endl;
  }
  return mode;
}

//Unread Character
bool QD25Base::SetUnreadCharacter(char *character){
  char command[100];

  if(strlen(character)>7){
    cout<<"Too long character!"<<endl;
    return false;
  }
  sprintf(command,"WR19#%s\r",character);

  if(!SendCommand(command)){
    cout<<"Set Unread Character Error!!"<<endl;
    return false;
  }
  
  return true;
}

int QD25Base::GetUnreadCharacter(){
  char command[100];
  char message[1024];

  sprintf(command,"RD19\r");

  if(!SendCommand(command,message)){
    return -1;
  }

  cout<<"Unread Character is \""<<message<<"\""<<endl;
  return 0;
}


bool QD25Base::SetDecodeTime(int mode){
  char command[100];

  sprintf(command,"WR1D#%02d\r",mode);

  if(!SendCommand(command)){
    cout<<"Set Decode Time Error"<<endl;
    return false;
  }
  return true;
}

int QD25Base::GetDecodeTime(){
  char command[100];
  char message[1024];
  int mode;

  sprintf(command,"RD1D\r");

  if(!SendCommand(command,message)){
    return -1;
  }

  cout<<"Decode Time:"<<message<<endl;
  mode = atoi(message);
  return mode;
}

bool QD25Base::SetDisplayReadDataMode(int mode){
  char command[100];

  sprintf(command,"WR34#%d\r",mode);

  if(!SendCommand(command)){
    cout<<"Set DisplayReadData Error!!"<<endl;
    return false;
  }
  return true;  
}


int QD25Base::GetDisplayReadDataMode(){
  char command[100];
  char message[1024];
  int mode;

  sprintf(command,"RD34\r");

  if(!SendCommand(command,message)){
    return -1;
  }

  mode = atoi(message);

  switch(mode){
  case 0:
    cout<<"Show Read Data through Connected Monitor"<<endl;
    break;
  case 1:
    cout<<"Don't Show Read Data"<<endl;
    break;
  }
  return mode;
}

bool QD25Base::SetFaleBeepMode(int mode){
  char command[100];

  sprintf(command,"WR35#%d\r",mode);

  if(!SendCommand(command)){
    cout<<"Set FaleBeepMode Error!!"<<endl;
    return false;
  }
  return true;  
}

int QD25Base::GetFaleBeepMode(){
  char command[100];
  char message[1024];
  int mode;

  sprintf(command,"RD35\r");

  if(!SendCommand(command,message)){
    return -1;
  }

  mode = atoi(message);

  cout<<"Fale Beep Mode:";
  switch(mode){
  case 0:
    cout<<"Beep ON"<<endl;
    break;
  case 1:
    cout<<"Beep OFF"<<endl;
    break;
  }
  return mode;
}

bool QD25Base::SetMarkPositionOutput(int mode){
  char command[100];

  sprintf(command,"WRA5#%d\r",mode);

  if(!SendCommand(command)){
    cout<<"SetMarkPositionOutput Error"<<endl;
    return false;
  }
  return true;
}

